
        ;       SJ64-CE (legacy) - used to be called as "!"

        ;       SDOS (v1.3) - C64 floppy speeder and disk utility
        ;       1986/2008/2016-2023 by Carroll/1570/Olessak
        ;       Public Domain (open-source and freeware)

        ;       VDOS is written by Edward Carroll (in 1986);
        ;       disassembled and modified for Jiffy protocol by 1570 (in 2008);
        ;       both sources are re-assembled by Robert Olessak (in 2016-2023).

        ;       SDOS is based on SJLOAD and VDOS. This is a common boot loader
        ;       that checks the drive and chooses among three possibilities:

        ;       1.) if JiffyDOS protocol is supported, it starts as SJLOAD;
        ;       2.) if not, it tries to start as VDOS (which is possible on
        ;           1541 and 1571 drives in C64 mode);
        ;       3.) if none of them is present, it stays at Kernal load.

        ;       SJ64 is a subset of SDOS: SJLOAD only (with VDOS code removed).
        ;       SV64 is a subset of SDOS: VDOS only (with SJLOAD code removed).

        ;       "CE" stands for "Compact Edition" (featureless and smaller).

        ;       Starting (in all cases):

        ;       LOAD"!*PROGRAM",8(,1) - (auto)start SDOS, fastload program
        ;       LOAD"!",8(,1) / RUN - (auto)start SDOS only

        ;       (on 1581 the * does not work, so please type = instead)

        ;       Usage (after having been installed):

        ;       VERIFY - send commands, read floppy status (not in CE!)
        ;       VERIFY"$" - display directory (only on the screen) (not in CE!)

        ;       LOAD"$" - load directory from last accessed device
        ;       LOAD"PROGRAM" - load program from last accessed device
        ;       LOAD"PROGRAM"(,8),1 - as above (as a machine code)
        ;       LOAD"PROGRAM"(,8),0 - as above (as a Basic code)

        ;       LOAD (without filename) => LOAD"?*",8(,1) (load first file)

        ;       The loader uses $02ae-$02ff and $cb00-$cfca as framework area
        ;       when active and $fb00-$ffca when hiding (under Kernal ROM).

        ;       New features (of SDOS in v1.0 to v1.2):

        ;       if using LOAD"!*PROGRAM",8(,1) the program will also be autorun
        ;       holding SHIFT key skips the autorun
        ;       holding C= key forces always VDOS (and skips drive check)
        ;       holding CTRL key forces always SJLOAD (and skips drive check)
        ;       holding both C= and CTRL keys at once forces always Kernal load
        ;       screen and IRQ are disabled while loading (in all three modes)
        ;       typing 0 or 1 for device number means latest with ,0 or ,1
        ;       typing no device number at all means latest once again
        ;       number 8 is substitute for last accessed device
        ;       specifying no filename means load first file
        ;       (plus several further minor bugfixes)

        ;       New features (of SDOS in v1.3):

        ;       no limit in loadable file size any more
        ;       loading even possible to RAM under the I/O
        ;       directory will also be fastloaded if possible
        ;       better optimization, device number, error handling etc.
        ;       (plus several further minor/major bugfixes)

        ;       at any time it can be re-installed by SYS737
        ;       at any time it can be un-installed by SYS700

        ;       memory swapping can be disabled by SYS727

        ;       (Normally the code resides in the "hidden" RAM under the Kernal
        ;       area, and is brought forth to $cb00 only during loading, then
        ;       afterwards swapped there back. So remains the $cb00-$cfca area
        ;       untouched and stays usable for any other purposes. You may
        ;       switch this feature off if you do not need it - to make things
        ;       faster, or to also use the "hidden" RAM.)

*=$0326

        ;       CHROUT and STOP vector <= Basic chaining + line number

        word    setup,$f6ed

        ;       63213 sys2061

        byte    $9e,$32,$30,$36,$31,$00,$00,$00

        ;       (* = $0332) <- (* = $080d) <- sys2061

        ;       ----------------------------------------
        ;       SETUP init (entry point for Basic)
        ;       ----------------------------------------

        sei

        ldy     #$15
        ldx     #$08

        jmp     btp+$04db

        ;       (* = $033a) <- (* = $0815)

        ;       "ui" (soft reset for drive identification)

uin     byte    $55,$49

*=$033c

        ;       (* = $033c) <- (* = $0817) <- autostart (sys828)

        ;       ----------------------------------------
        ;       SETUP init (entry point for autostart)
        ;       ----------------------------------------

setup   jsr     $ff8a

retup   jsr     reset
        jsr     rwap

        ldy     #$00
        ldx     #$05

stp     lda     $0400,y
        sta     $cb00,y
        iny
        bne     stp

        inc     stp+2
        inc     stp+5
        dex
        bne     stp

        jsr     drive
        jsr     $02ce

        lda     #<scrn
        ldy     #>scrn
        jsr     $ab1e

        jsr     $a644

        dec     $b7
        bmi     basc

        ldx     $2b
        ldy     $2c

        jsr     $ffd5
        bcs     basc

        stx     $2d
        sty     $2e

        jsr     $a533

atp     lda     #$00
        ldx     #$05
        ora     $028d
        and     #$01
        bne     basc

        stx     $c6

rtp     lda     run-1,x
        dex
        sta     $0277,x
        bne     rtp

        ;       basic warm start

basc    jmp     ($a002)

*=$0398

        ;       (scnclr)

scrn    byte    $99,$93,$0e,$00

*=$039c

        ;       "run:" (into keyboard buffer)

run     byte    $52,$55,$4e,$3a,$0d

*=$03a1

        ;       ----------------------------------------
        ;       SJLOAD/VDOS helper ($02ae-$02ff)
        ;       ----------------------------------------

        ;       "?*" (filename at $02ae)

s02ae   byte    $3f,$2a

        ;       (swapping forth)

s02b0   jsr     $02ea

        ldy     #$60
        cpy     $cfca
        bne     s02ea

        cli
        rts

        ;       sys700 (jsr $02bc): un-install (by restoring Kernal)

s02bc   jmp     $ff8a

        ;       (* = $02bf) <- Kernal load ($02c5-$02c6) = LOAD vector (orig)
        ;       (i.e. swapping back, calling load, then swapping forth again)

s02bf   tya
s02c0   jsr     $02ea
s02c3   sei
s02c4   jsr     $f4a5
        jsr     $02ea
        jmp     $cf40

        ;       sys717 <- sys737 (re-install)

s02cd   clc
s02ce   jsr     $02b0

        ;       re-make CRC etc.

s02d1   jsr     sweax+$c700

        clc
        bcc     s02ea

        ;       sys727 (jsr $02d7): re-install with swapping disabled
        ;       (speedloader code will stay for ever fixed at $cb00)

s02d7   jsr     $02b0

s02da   ldy     #$60
        sty     $02ea
        bpl     s02d1

        ;       sys737 (jsr $02e1): re-install

s02e1   clc
        bcc     s02ce

        ;       (NOP at $02e3 hidden in the operand!)

        jsr     $02b0
        jsr     $cc29

        ;       (swapping back)

s02ea   nop
        nop

s02ec   pha
s02ed   tya
        ldy     #$34
        sei
        pha
        sty     $01
s02f4   jsr     $ffcb
        ldy     #$37
        pla
        sty     $01
        tay
s02fd   pla
s02fe   cli
s02ff   rts

*=$03f3

        ;       ----------------------------------------
        ;       SETUP subroutines
        ;       ----------------------------------------

        ;       drive reset and installing helper

reset   ;sei

        ldy     #$00
        lda     #$80

        sty     $90
        sta     $9d
        ;sty     $b8
        sty     $b9

        sty     $c6

        ;       checking for * or = in filename (to load another file if so)

silp    dec     $b7
        beq     lasc

        lda     ($bb),y

        inc     $bb
        bne     silc

        inc     $bc

silc    cmp     #$2a
        beq     sasc
        cmp     #$3d
        bne     silp

sasc    inc     $b7

        ;       checking for modified LOAD vector

lasc    lda     $0330
        ldx     $0331
        bpl     crchk

        sta     lcf8b+1
        stx     lcf8d+1

        sta     s02c4+1
        stx     s02c4+2

        ;       checking for DTV

        cpx     #$f7
        bne     crchk

        ;       (self-modification code for DTV)

        ldx     #$19
lchlp   lda     tce0a-1,x
        dex
        sta     lce09+1,x
        bne     lchlp

        ;       turn off all fast modes (and check) <- (required for timing)

crchk   jsr     lce09
        jsr     lcf12

        ;       ($09a3 = lcfa3) <- $cfa3 before copying (value of $d030)

        sta     lcfa2+1

        ;       (* = $0441)

        ;       checking for modified LOAD vector and machine type

check1  ;sei

        ;       detect DTV

        txa
        beq     crock

        ;       detect MEGA65 using POKE 0,64

        lda     #$40
        ldx     $00
        sta     $00
        cpx     $00
        stx     $00
        beq     qrock

        tya

        ;       CPU check ($1a = INC A on 4510 or 65816, but NOP on 6510 CPU)

        byte    $1a
        beq     chock

        ;       CPU check ($eb = NOP on 65C02, XBA on 65816, ROW $AA98 on 4510)

        byte    $eb
        tya
        tax

        ;       4510 (C65 or MEGA65) found -> normal load (like NTSC/DTV)

        asl
        bne     qrock

        byte    $eb

        ;       65C02 found -> Kernal LOAD (not suitable for timing)

        tay
        ;beq     chyck

        ;       65816 (SuperCPU) found -> turbo off <- (required for timing)
        ;       (not necessary here when already done above)

        ;sta     $d07a

        bne     chyck

        dex
        byte    $2c

        ;       self-modification on C65/MEGA65 (device number for Kernal LOAD)

qrock   ldx     #$0a

        stx     lccc6+1
        ;stx     c65ck+1
        ;stx     c65cx+1

        ;       $cf10 <- $ce24 (RTS) <-> $ce09 (on C65/MEGA65 or DTV)

crock   ldy     #$09
        sty     lcf0f+1

chock   ;iny
chyck   ;sty     $aa

        ;       last accessed device

        lda     $ba
        bne     haba

        lda     #$08
        sta     $ba

haba    ldx     #$58

        sta     $a8
        sta     $a9

hlp     lda     $68,x
hlx     sta     $0960,x
        lda     s02ae-$0007,x
        sta     $02a7,x
        dex
        bpl     hlp

        stx     $90

        ;       checking for C= or CTRL depressed

check2  ;jsr     $ff9f

        lda     $028d

;        ldx     $ba
        ora     atp+1

;c65ck   cpx     #$04
        sta     atp+1
;        bcs     orck

;c65cx   ldx     #$08

orck    and     #$06
        cmp     #$06
        bcc     hlx+1

        ;       self-modification (revert to Kernal LOAD)

oerl    rts

        ;pla
        pla

        ;       self-modification (from VDOS to KERNAL)

kerl    ldy     #$ff

        ;       (always branch on BCC if Kernal LOAD)

        sty     $ccc7

        ;bmi     xwap

        ;ldy     #$cc
        ;sty     swecc+$c701

        bmi     xwap

        ;       drive check

drive   lda     #$68
        ;lda     #$a0

        ldy     #$cb

        sta     oerl
        ;sta     kerl

        sty     swc+$c701
        sty     swync+$c701

        ldy     #$fb
        lda     #$cf
        sty     swd+$c701
        sta     swynd+$c701

        ;       turn off fast mode + save registers

        jsr     $cc20

        ;ldy     #$00
        ;jsr     $ceff

        ;       make CRC (of $02b0-$02ff + $cbf8-$cc47 + $cfcb-$cff5 at first)

        jsr     sweax+$c700

        ;       checking for C= or CTRL depressed

        jsr     check2

        ;       swapping $c000 and $f000 (back)

xwap    jsr     $ffe7

xlp     ldx     #$58
zlp     lda     $0960,x
        sta     $68,x
        dex
        bpl     zlp

        txa

        ;       release IEC bus (on C64)
        ;       (POKE 56576,199)

        ;ldy     #$c7
        ;sty     $dd00

        ;       return from load routine and/or
        ;       restore screen + fast mode (before Kernal call) etc.

        jsr     $cf40

        sta     swex+$c702

        ;       swapping $c000 and $f000 (back)

        ;jsr     rwap

        ;       sys717 <- sys737 (re-install) <- sweax

        ;jmp     $02ce

        ;       swapping $c000 and $f000

rwap    sei

        ldx     #>swp
        lda     #$34
        stx     s02f4+2

        jmp     rwax

        ;       (* = $ce0a-$ce22) <- DTV code (optional)

tce0a   byte    $01
        sta     $d03f

        byte    $32,$99

        tya
        beq     tce15

        lda     #$03

tce15   byte    $32,$00

        tya
        beq     tce1c

        lda     #$20

tce1c   sta     $d03c

        lda     #$00
        byte    $8d,$3f

*=$04f8

l600

        ;       (* = $cbf8)

        ;       ----------------------------------------
        ;       common helper ($cbf8-$cc1f)
        ;       ----------------------------------------

        ;       display message, install load vector etc.

sweax   ldy     #<$02e3
        ldx     #>$02e3

        sty     $0330
        stx     $0331

        ;       re-make CRC

        jsr     $cfb1

        ;       make CRC (of $02b0-$02ff + $cbf8-$cc47 + $cfcb-$cff5 at first)
        ;       (or $ffcb-$fff5 later)

        bit     $02ea
        sta     $cf67
        bmi     sweay

        ;       self-modification (for bad CRC of swap r.) <- if no swapping

swecc   lda     #$cc
        ;lda     #$cb
        inx

        ;       (and modify controlling data pointer from $cb/$fb to $cc/$fc
        ;       except on VDOS where it must stay $cb as can be seen here
        ;       since VDOS needs its routines at the $cb00-$cbff area)

        sta     $ced7
        eor     #$30
        sta     $cefb

sweay   ldy     #$37
        stx     $cf76
        sty     $01

lcc1f   rts

*=$0520

        ;       (* = $cc20)

        ;       ----------------------------------------
        ;       common LOAD/VERIFY routines ($cc20-$ccff)
        ;       ----------------------------------------

        ;       (* = $cc20) <-> (* = $ceff) <- turn off fast mode + save reg.

lcc20   lda     $d020
        sta     $cf9e

        jmp     $cf03
        ;jmp     $cf05

        ;       load/verify

lcc29   sta     $93

        lda     $ba

        cmp     #$08
        beq     lcc37
        cmp     #$02
        bcs     lcc39

        ;and     $b9
        sta     $b9

        ;       last accessed device

lcc37   lda     $a8

lcc39   and     #$0f
        bne     lcc3f

        lda     #$08

lcc3f   ldy     #$00
        ldx     #$0b
        cmp     #$04

        sty     $90
        sta     $a8
        sta     $ba

        ;       turn off fast mode + save registers

        jsr     $cc20
        ;jsr     $ceff

        sta     $cfa3

        ;       (checking if "$" and/or LOAD with or without filename)

        lda     $b7
        ;sty     $d015
        bne     lcc64

        lda     #<$02ae
        sta     $bb
        lda     #$02
        sta     $bc
lcc5d   bcs     lcc62

        lsr
        inc     $bb

lcc62   sta     $b7

lcc64   lda     ($bb),y

        eor     #$24
        bne     lcc6c

        ;       (checking if "$" and/or LOAD with or without filename)

        sty     $b9

        ;       turn off screen

lcc6c   stx     $d011
        sty     $d020

        ldx     $ba

        ;       (*+1 = $cc75)

lcc74   cpx     #$08

        ldx     $b9
        sty     $a7
        sta     $aa
        stx     $ab

        ldy     $93
        bne     lcc84

        ;       (* = $cc82)

lcc82   bcs     lsnc0

        ;       Kernal LOAD (orig)

lcc84   jmp     $02bf

        ;       (* = $cc87) <-> $ccef (see it there!)

lcc87   jsr     $ccef

        bit     $90
        bpl     lcc1f

lcc8e   pla
        pla

        ;       device not present error

lcc90   jsr     $cf40
        jmp     $f707

        ;       raster synchronization (having the screen turned off) #1

lsnc0   bit     $d011
        bpl     lsnc0

;lsnc1   bit     $d011
;        bmi     lsnc1

        ;       "searching for..."

        jsr     $f5af

        ;       open file (with error handling) + talk
        ;       (if fails: device not present error)

lcc9e   jsr     $cc87

        ;       start address (lb, hb)

        jsr     $ffa5
        tay

        ;       (if fails: file not found error)

        lda     #$bf
        and     $90
        bne     lccd7

        jsr     $ffa5
        ;sei

        ldx     $ab
        bne     lccb6

        ldy     $c3
        lda     $c4

lccb6   sty     $ae
        sta     $af

        ;sty     $ac
        ;sta     $ad

        ;       control data pointer (set forth)

        jsr     $ced4

        ;       (second try!)

        ldx     $a7
        bne     lcce5

        ;       "loading" (print message)

        jsr     $f5d2

        ;       checking for device number again, for Kernal LOAD
        ;       (on C65/MEGA65 where it must be $0a instead of $08)

        lda     $ba

        ;       (or $ff to always branch on BCC if Kernal LOAD)

        ;       (*+1 = $ccc7)

lccc6   cmp     #$08
        ldy     $aa
        stx     $aa
        bcc     lcce5
;lccce   beq     lcce5
lccce   bcc     lcce5

        sta     $a7

        ;       VDOS <-> SJLOAD (start loading) <- start over (or continue)

        jsr     $ce25

        ;       if speedloading failed, we try again with Kernal LOAD

        bcs     lcc9e
        ;bcc     lccef+1

        ;       file not found error

lccd7   jsr     $cf40
        jmp     $f704

        ;       Kernal LOAD (open), fetching byte by byte

lccdd   jsr     $ffa5
        jsr     $cec7
        stx     $01

lcce5   ldy     $90
        beq     lccdd

        ;       control data pointer (set back)

lcce9   jsr     $cef8

        ;       loading is ready -> jump to return from load routine

lccec   jmp     $cf30

        ;       (* = $ccef) <-> $cc87 (see it there!)

lccef   lda     #$60
        sta     $b9

        ;       (open) <- it aborts if device not present!

        jsr     $f3d5

        ;       (talk)

lccf6   lda     $ba
        jsr     $ffb4
        lda     $b9
        jmp     $ff96

*=$0600

        ;       ----------------------------------------
        ;       SJLOAD ($cd00-$cdff)
        ;       ----------------------------------------

        ;       (* = $cd00) <- talk

ltalk   ora     #$40

        ;       (* = $cd02)

lsendb  sta     $95
        jsr     $ee97

        cmp     #$3f
        bne     lca1

        jsr     $ee85

lca1    jsr     $ce00

        ;       (* = $cd11) <- send IEC byte

lwiecs  jsr     $ee8e
        jsr     $ee97
        jsr     $eeb3

        ;       device not found (but jump to timeout here instead)

        jsr     $eea9
        bcs     ltmouc

lcont1  jsr     $ee85

lcont4  jsr     $eea9
        bcc     lcont4

        jsr     $ee8e

        txa
        pha

        ldx     #$08
        bpl     lsdbts

*=$0630

        ;       (* = $cd30) <- 32 bytes trashed by Kernal (jsr $ff8a)

rwax    ldy     #$60
        sta     $01

        sty     $fff5
        cpy     $fff5
        beq     swax

        ldx     #$df
        sty     $02ea
        sta     lcc5d
        stx     lceed+1
        sty     swp
        sty     rwap

swax    jmp     s02ea

*=$0650

        ;       (* = $cd50)

lsdbts  cmp     ($00,x)

        bit     $dd00
        bpl     ltbout

lcont5  jsr     $ee97

        ror     $95
        bcs     lci1

        jsr     $eea0

lci1    jsr     $ee85

        lda     $dd00
        and     #$cf
        ora     #$10
        sta     $dd00

        ;       send two bits

        and     #$08
        beq     ltwbst

        lda     $95
        ror
        ror

        cpx     #$02
        bne     ltwbst

        ldx     #$1e

lwack1  bit     $dd00
        bpl     lwack2

        dex
        bne     lwack1
        bpl     lcont6

ltbout  pla
        tax

        ;       timeout

ltmout  sec

ltmouc  pla
        pla

        jmp     $edb0

lwack2  bit     $dd00
        bpl     lwack2

        ;       when we are here JD is present in floppy

lcont6  ldx     #$02

ltwbst  dex
        bne     lsdbts

        ldx     #$56

lcont7  dex
        beq     ltbout

        lda     $dd00
        bmi     lcont7

ltbok   pla
        tax

        rts

        ;       (* = $cda5) <- send secondary address

lsndsa  sta     $95
        jsr     $cd11

        lda     #$23
        sta     $dd00

lwca1   bit     $dd00
        bvs     lwca1

        rts

        ;       (* = $cdb5) <- iecin (jumped from $ee13)

ll3     lda     $dd00
        cmp     #$40
        bcc     ll3

        sei
        sei
        cmp     ($00,x)
        cmp     ($00,x)
        lda     #$03
        sei
        sei
        sta     $dd00
        cmp     ($00,x)
        cmp     ($00,x)
        ora     $dd00
        lsr
        lsr
        sei
        ora     $dd00
        lsr
        lsr
        eor     #$03
        eor     $dd00
        lsr
        lsr
        eor     #$03
        sei
        eor     $dd00

        pha
        lda     #$23
        bit     $dd00
        sta     $dd00
        bvc     lend1
        bpl     lerr1

        pla
        lda     #$42
        jmp     $edb2

lerr1   lda     #$40
        jsr     $fe1c

lend1   pla
        clc

        rts

*=$0700

        ;       ----------------------------------------
        ;       common LOAD routines ($ce00-$cfca)
        ;       ----------------------------------------

        ;       (* = $ce00)

lce00   lda     #$08
        ora     $dd00
        sta     $dd00

        rts

        ;       (* = $ce09) <- C65/MEGA65 code (normal)

lce09   lda     #$a5
        sta     $d02f

lce0e   lda     #$96
        sta     $d02f

        lda     $d031
        and     #$bf

        dey
        bmi     lce1d

        ora     #$40

lce1d   iny
        sta     $d031

lce21   sty     $d02f

lce24   rts

        ;       (* = $ce25) <- VDOS/SJLOAD (common) <- start loading

        ;       raster synchronization (having the screen turned off) #2

lce25   bit     $d011
        bmi     lce25

        lda     $dd00
sce2d   pha

sce2e   ;jsr     $ffab

        tya
        pha
        beq     sceq

        ;jsr     $ce94
        ;jsr     $ce79

        inc     $b9

        ;       (talk) + send secondary address

sceq    jsr     $ce94

        ;       (jump to $cea6) <- LOAD "$" (with SJLOAD) byte by byte

        pla
        sei
        ;beq     sce2e+$0078
        beq     scea6

        dec     $b9

        ldy     $90
        ;bne     sce2e+$007c
        bne     llpdrsg

        ;       start loading

llnrlp  lda     #$03
lwch0   sta     $dd00

lwch1   bit     $dd00
        bvc     lwch1

        ;bmi     sce2e+$007c
        bmi     llpdrsg

ltrblk  bit     $dd00
        bpl     ltrblk

ltrbyt  sei
scrts   lda     ($60,x)

        ;       (* = $ce55)

ltrbyx  ldx     #$23
ltrbax  lda     #$03
        stx     $dd00

        bit     $dd00
        bvc     lwch0
        ;bvc     llnrlp

        sei
        sta     $dd00
        lda     $dd00
        sei
        lsr
        lsr
        eor     $dd00
        bit     $00
        lsr
        lsr
        eor     $dd00
        bit     $00
        lsr
        lsr
        eor     $dd00
        eor     #$03

        bit     $af
        bmi     lups

        sta     ($ae),y

        inc     $ae
        bne     ltrbyt

        inc     $af
        ;bne     lupx
        bne     ltrbyx
        ;bne     ltrbyt

lups    jsr     $cec8
        stx     $01

lupx    jmp     ltrbyx+$c700
        ;jmp     $ce55

;        ;       (* = $ce79) <- untalk
;
;sce79   jsr     $ce00
;
;        jsr     $ee8e
;        lda     #$5f
;        jsr     $cd02
;        jsr     $edbe
;
;        txa
;        ldx     #$0a
;
;ll2     dex
;        bne     ll2
;
;        tax
;
;        jsr     $ee85
;        jmp     $ee97

        ;       (* = $ce94) <- talk

sce94   lda     $ba
        jsr     $cd00
        lda     $b9
sce9b   jmp     $cda5

        ;       (* = $ce9e)

        ;       SJLOAD (open), fetching byte by byte

sce9e   jsr     $cdb5
        jsr     $cec7
        stx     $01

        ;       (* = $cea6) <- $ce2e + $0078

scea6   ldy     $90
        beq     sce9e
        ;beq     scea6-$0008

llpdrsg

        ;       (* = $ceaa) <- $ce2e + $007c

sceaa   lda     $90
        and     #$bf
        bne     scebb

        ldx     #$64
        clc

lwok1   bit     $dd00
        bvc     scebc

        dex
        bne     lwok1

scebb   sec

scebc   php

        ;       control data pointer (set back)

        jsr     $cef8

        ;       (untalk)

        ;jsr     $ce79

        ;       loading is ready -> return from load routine

scec0   plp

        ;       (* = $cec1)

        ;       VDOS/SJLOAD (common) <- end of loading (restore $dd00 + close)

lcec1   pla
        sta     $dd00
        bne     lcf2c

        ;       (* = $cec7)

lcec7   sei

        ;       (*+1 = $cec9) <- write into RAM everywhere ($35: + I/O area on)

lcec8   ldx     #$34
        stx     $01

lcecc   ldx     #$36

        ;       (* = $cece) <- increment data pointer

lcece   sta     ($ae),y

lced0   inc     $ae
        beq     lcedb

        ;       control data pointer ($cb00-$cfca and $fb00-$ffca swapped!)
        ;       (or $cc00/$fc00 if no swapping)

lced4   lda     $af

        ;       (*+1 = $ced7)

        cmp     #$cb
        bcs     lcedf

        rts

lcedb   inc     $af
        bne     lced4

lcedf   cmp     #$ff
        lda     $ae
        bcc     lcee9

        cmp     #$cb
        bcs     lcef1

lcee9   cmp     #$cb
        lda     $af
lceed   sbc     #$cf
        bcs     lcef7

lcef1   lda     $af

lcef3   eor     #$30
        sta     $af

lcef7   rts

        ;       control data pointer ($cb00-$cfca and $fb00-$ffca swapped!)
        ;       (or $cc00/$fc00 if no swapping)

lcef8   lda     $af

        ;       (*+1 = $cefb)

        cmp     #$fb
        bcs     lcef3

        rts

        ;       (* = $ceff) <- (* = $07ff)

        byte    $00

        ;       (* = $cf00) <- (* = $0800) <- patching for Basic (NEW)

        byte    $00,$00,$00

lcf03   ldy     #$00

;        ;       (* = $ceff) <- turn off fast mode + save registers
;
;lceff   lda     $d020
;        sta     $cf9e

lcf05   lda     $d011
        sta     $cfa8

        ;       (* = $cf0b) <- turn off fast mode

lcf0b   lda     #$37
        sta     $01

        ;       (*+1 = $cf10) <- $ce24 (RTS) <-> $ce09 (on C65/MEGA65 or DTV)

lcf0f   jsr     $ce24
        ;jsr     $ce21
        ;sty     $d02f

        ;       (* = $cf12)

lcf12   lda     $d030
        sty     $d030
        sty     $d07a

lcf1b   rts

        ;       (* = $cf1c) <- (listen to command channel)

lcf1c   lda     $ba
        jsr     $ffb1
        lda     #$6f
        jmp     $ff93

        ;       (* = $cf26) <- (untalk + close)

lcf26   jsr     $ffab
        jmp     $f642

        ;       close on success (otherwise retry)

lcf2c   bcs     lcf1b

        pla
        pla

        ;       (* = $cf30) <- (return from load routine with close)

lcf30   jsr     $cf26

        ;       (* = $cf33) <- send "i" to device after loading (if no error)
        ;       (needed by VDOS to get rid of the loader code in drive)

lcf33   jsr     $cf1c
        lda     #$49
        jsr     $ffa8
        jsr     $ffae

        ;       (* = $cf3e)

        sei
        clc

        ;       (* = $cf40)

        ;       return from load routine and/or
        ;       restore screen + fast mode (before Kernal call) etc.

lcf40   pha

        ;       EOI cleared on status (checking for an error)

        lda     #$be
        ;lda     #$bf
        ;lda     #$ff
        and     $90
        sta     $90
        beq     lcf4a

        sec

        ;       if succeeded, stores the last used device number ($a8 -> $a9),
        ;       but if failed, then reverts to the former ($a8 <- $a9)

lcf4a   lda     $a8
        bcc     lcf52

        lda     $a9

lcf50   sta     $a8
lcf52   sta     $a9

        php
        jsr     $f32f
        ;jsr     $ffe7
        plp

        ;       release IEC bus (on C64)
        ;       (POKE 56576,199)

        lda     $dd00
        and     #$03
        ora     #$c4
        sta     $dd00

        jsr     $cfb1

        ;       (*+1 = $cf67) <- CRC of $02b0-$02ff + $cbf8-$cc47 + swap r.

        eor     #$00
        beq     swend

        pla
        tay

        ;       (* = $cf6c)

        pla
        pla

        tya
        pha

        txa
        ldy     #>$cc29
        ldx     #<$cc29

        ;       (*+1 = $cf76) <- CRC of swap routine ($ffcb-$fff5)

        eor     #$00
        stx     $cf6c
        bne     swenc

swexx   lda     $cf87,x
        dex
        sta     $0100,x
        bne     swexx

        jmp     $0100

        ;       (* = $cf88)

swaff   jsr     $ffcb

        ;       (*+1 = $cf8c) <-> (*+3 = $cf8e) <- Kernal LOAD vector (orig)

lcf8b   ldx     #<$f4a5
lcf8d   ldy     #>$f4a5

        ;       (* = $cf8f)

swenc   stx     $0330
        sty     $0331

        ;       (* = $cf95) <- return from load routine

swend   lda     #$37
        ldx     $ae
        ldy     $af
        sta     $01

        ;       restore screen + fast mode (before Kernal call)

        ;       (*+1 = $cf9e)

        lda     #$00
        sta     $d020

        ;       (*+1 = $cfa3)

lcfa2   lda     #$00
        sta     $d030

        ;       (*+1 = $cfa8)

        lda     #$1b
        and     #$7b
        sta     $d011

        pla
        cli

        ;       (* = $cfb0)

        rts

        ;       (* = $cfb1) <- make CRC

lcfb1   sei

        lda     #$34
        ldx     #$2a
        ldy     #$4f
        sta     $01

swex    eor     $cfcb,x
        dex
        bpl     swex

        tax

swfx    eor     $02b0,y
        eor     l600+$c700,y
        dey
        bpl     swfx

        ;       (* = $ffca) <- (* = $cfca)

        rts

        ;       (* = $08cb) <- (* = $cfcb)

        ;       ----------------------------------------
        ;       SJLOAD/VDOS helper ($ffcb-$fff5)
        ;       ----------------------------------------

        ;       (* = $ffcb) swapping $cb00-$cfca and $fb00-$ffca
        ;       (also for swapping $c000-$cfff and $f000-$ffff)

swp     ldy     #$00

swc     lda     #$c0
        sty     $0c
        sta     $0d

swd     lda     #$f0
        sty     $0e
        sta     $0f

swy     lda     ($0c),y
        pha
        lda     ($0e),y
        sta     ($0c),y
        pla
        sta     ($0e),y

        iny
        tya
        bne     swync

        inc     $0d
        inc     $0f

swync   eor     #$00
        bne     swy

swynd   lda     #$d0

        eor     $0d
        bne     swy

rcrts   rts

        ;       (* = $fff6) <- (* = $cff6)

        plp
        sei
        php
        rti

        ;       (* = $fffa) <- (* = $cffa)

        word    $fff6
        word    $fff6
        word    $fff6

        ;       (* = $0900) <- (* = $d000)

*=$0900

        ;       (* = $0900) <- (* = $0ddb) <- Basic start (sys2061)

        ;       ----------------------------------------
        ;       SETUP init (entry point for Basic)
        ;       ----------------------------------------

btp     lda     $0800,y
        sta     $0325,y
        iny
        bne     btp

        inc     btp+$04dd
        inc     btp+$04e0
        dex
        bne     btp

        jmp     retup
        ;jmp     setup

        ;       (* = $0915) <- (* = $0df0)

        ;       compiled with CBM prg Studio v3.13 (by Arthur Jordison)
        ;       http://ajordison.co.uk/

        ;       Rosetta Interactive Fiction project homepage:

        ;       http://istennyila.hu/rosetta

        ;       v0.13 (c) 2012-2023 by Robert Olessak

        ;       SDOS project page:

        ;       http://istennyila.hu/sdos
