
        ;       SDOS - VIC-20 floppy speeder and disk utility

        ;       SDOS (v1.3) - written by Robert Olessak in 2019-2023, based on:

        ;       SJVIC (version 09) - SDOS Edition (2019):

; C64 floppy speeder and disk utility based on VDOS by Edward Carroll 1986
; modified for Jiffy LOAD compatibility by 1570 in 2008
; modified for VIC-20 and for Jiffy SAVE by Diddl in 2009
; V06 version for VIC-20 and for basic I/O by Diddl in 2010
; V07 version for VIC-20 by nbla000 in 2010
; V08/09 versions (fixed and optimized) by Robert Olessak in 2019

        ;       "SDOS Edition" (09) derives from my "Compact Edition" (08):

        ;       "Compact Edition" means I first removed all the surplus extras
        ;       (like print messages etc.) from V07, then further optimized the
        ;       code to get as small as possible (256 bytes less!).

        ;       Plus a little bugfix: 1.) it first checks for the presence of a
        ;       Jiffy or an NTSC Kernal, and if any found, does not initialize;
        ;       2.) does not overwrite the memory at $8000-$9fff (I/O area).

        ;       Automatic code relocation:

        ;       This version of SDOS requires six pages of memory (1.5K), which
        ;       is by default at $ba00-$bfff. If it is not available, then the
        ;       next tried location is at $a000-$a5ff; then at $0a00-$0fff or
        ;       $0400-$09ff; then $9a00-$9fff (within I/O) or $7900-$7eff.
        
        ;       If none of them succeeds, it rather goes into the end of Basic.

        ;       C= key forces to choose $9a, Ctrl $0a, or both together $a0.

        ;       Any of the two keys also forces to skip drive check. If either
        ;       is pressed alone, it forces to choose SJLOAD, while both
        ;       together forces to choose Kernal load.

*=$0326

        ;       CHROUT and STOP vector <= Basic chaining + line number

        word    main,$f770

        ;       63344 syspeek(44)*256+21

        byte    $9e,$c2,$28,$34,$34,$29,$ac,$32,$35,$36,$aa,$32,$31,$00,$00,$00

        ;       (* = $0155) <- (* = $0415) <- (* = $033a) <- start (sys826)

        ;       ----------------------------------------
        ;       SETUP init (entry point for Basic)
        ;       ----------------------------------------

        ldy     #$5f
        sei

slp     lda     ($2b),y
        dey
        sta     $0142,y
        bne     slp

        jmp     $0163

        ;       (* = $0163) <- (* = $0423) <- (* = $0348)

        lda     $2c
        cmp     #$14
        ldy     #$7b
        ldx     #$09
        bcs     dow

        ;       (* = $016d)

upw     txa
        adc     $2c
        sta     $0175

        ;       (* = $0173) <- upward relocation

clp     lda     $0d00,y
        sta     $1c25,y
        dey
        cpy     #$ff
        bne     clp

        dec     $0175
        dec     $0178
        dex
        bpl     clp

qlp     jmp     bain+$1000

        ;       (* = $018a)

dow     sta     $018f

        ;       (* = $018d) <- downward relocation

blp     lda     $0400,y
        sta     $1325,y
        iny
        bne     blp

        inc     $018f
        inc     $0192
        dex
        bne     blp

        bpl     qlp

        ;       (* = $1386) <- (* = $0386) <- (* = $0461) <- autostart

        ;       ----------------------------------------
        ;       SETUP init (entry point for autostart)
        ;       ----------------------------------------

main    sei
        ldy     #$a0
        ldx     #$0a

mlp     lda     $0300,y
        sta     $1300,y
        iny
        bne     mlp

        inc     $038d
        inc     $0390
        dex
        bne     mlp

        jmp     bain+$1000

        ;       (* = $13a0) <- (* = $03a0) <- (* = $047b)

        ;       ----------------------------------------
        ;       SETUP init (after relocation)
        ;       ----------------------------------------

bain    jsr     $ff8a

        ldy     #$4f
salp    lda     sain+$0fff,y
        dey
        sta     $0141,y
        bne     salp

        jmp     $0141

        ;       (* = $0141) <- (* = $13b1)

sain    cpy     crc+$1000
        sbc     nix+$1000
        bne     new

        jsr     setup+$1000

new     jsr     $c644

        sei
        ldy     #$00

        ;       checking for * in filename (if not yet done)

filp    dec     $b7
        bmi     basc

        ;       (* = $0156) <- BPL FILX (self-modification)

        lda     ($bb),y

        inc     $bb
        bne     filc

        inc     $bc

filc    eor     #$2a
        bne     filp

filx    ldx     $2b
        ldy     $2c

        jsr     $ffd5
        bcs     basc

        stx     $2d
        sty     $2e

        jsr     $c533

        ;       (*+1 = $0173) <- checking for SHIFT pressed

        lda     #$00
        ldx     #$05
        ora     $028d
        and     #$01
        bne     basc

        stx     $c6

rtp     lda     $018a,x
        dex
        sta     $0277,x
        bne     rtp

        ;       basic warm start

basc    jmp     ($c002)

        ;       (* = $018b) <- (* = $13fb) <- "run:" (into keyboard buffer)

run     byte    $52,$55,$4e,$3a,$0d

*=$0400

        ldx     #$16

        ;       (*+2 = $0404)

wlp     lda     wtb-1,x
        dex
        sta     $031e,x
        bne     wlp
        rts

        ;       (* = $040c) <- "ui" (soft reset for drive identification)

uin     byte    $55,$49

        ;       (* = $040e)

wtb     word    jchkin,jckout,jclrch,jbasin,jbsout,$f770,jgetin,jclall,$fed2
        word    myload,mysave

*=$0424

        ;       ----------------------------------------
        ;       SYS procs SAVE
        ;       ----------------------------------------

        ;       old save proc

sold    jmp     $f685

        ;       SJ Save vector:

msave   ldx     $ba
ncpk1   cpx     #$08
        bcc     sold

        ;       checking for Jiffy Kernal

        ldy     #$4a
        cpy     $e438
        beq     sold

        ;       channel

        lda     #$f1
r00     jsr     diecnm
        bcs     mysrr

        lda     #$61
r01     jsr     disk_listen

        ;       $c1/$c2 --> $ac/$ad

        jsr     $fbd2

        lda     $ac
r02     jsr     iecout
        lda     $ad
r03     jsr     iecout

        ;       print "saving"

        jsr     $f728

        ldy     #$00

mys00   jsr     $fd11
        bcs     myse0

        lda     ($ac),y
r04     jsr     iecout

        jsr     $ffe1
        bne     mys02

r05     jsr     myse0
        jmp     $f6ce

        ;       increment address

mys02   jsr     $fd1b
        bne     mys00

r06
myse0   jsr     unlisten
r07     jsr     disk_close_sa

        clc
mysrr   rts

        ;       ----------------------------------------
        ;       JIFFY procs
        ;       ----------------------------------------

r08
diecnm  jsr     disk_listen

        ;       put name to iec and unlisten

iecnamout

        lda     $90
        bmi     dmerr1

        ldx     $b7
        beq     dmok2

        ldy     #$00

dm2     lda     ($bb),y
r09     jsr     iecout
        iny
        dex
        bne     dm2

r10
dmok2   jsr     unlisten

dmok    clc
        rts

        ;       device not present error

dmerr1  jmp     $f78a

disk_listen

        pha

        lda     #$00
        sta     $90

        beq     dili2

disk_listen_2

        pha

dili2   lda     $ba
r11     jsr     listen

        pla
r12     jsr     listensa

dita5   lda     $90
        bpl     dmok

        sec
        rts

        ;       Jiffy listen:

talk    ora     #$40
        byte    $2c

listen  ora     #$20
        jsr     $f160

lee1c   pha
        bit     $94
        bpl     l6e2b

        sec
        ror     $a3
r13     jsr     new_iecout

        lsr     $94
        lsr     $a3

l6e2b   pla
        sta     $95

        sei
        lda     #$00
        sta     $a3
        jsr     $e4a0

        cmp     #$3f
        bne     l6e38

        jsr     $ef84

r14
l6e38   jsr     ora80

lee40   jsr     $ef8d

        jsr     $e4a0
        jsr     $ef96

old_iecout

        ;       old iec out

        sei
        jsr     $e4a0
        jsr     $e4b2
        lsr
        bcs     l6eb4

        jsr     $ef84
        bit     $a3
        bpl     l6e66

l6e5a   jsr     $e4b2
        lsr
        bcc     l6e5a

l6e60   jsr     $e4b2
        lsr
        bcs     l6e60

l6e66   jsr     $e4b2
        lsr
        bcc     l6e66

        jsr     $ef8d

        txa
        pha
        ldx     #$08

l6e73   lda     $911f
        and     #$02
        bne     l6e7f

        ;       timeout

        pla
        tax
        jmp     $eeb7

l6e7f   jsr     $e4a0
        ror     $95
        bcs     l6e89

        jsr     $e4a9

l6e89   jsr     $ef84

        lda     $912c
        and     #$dd
        ora     #$02

        php
        pha
r15     jsr     lf96e
        pla
        plp

        dex
        bne     l6e73

        pla
        tax
        jmp     $eea0

ora80   lda     #$80
        ora     $911f
        sta     $911f
        jmp     $ef8d

        ;       device not present error

l6eb4   jmp     $eeb4

        ;       timeout

l6eb7   jmp     $eeb7

lf96e   sta     $912c

        bit     $911f
        bpl     lf997

        cpx     #$02
        bne     lf997

        txa
        ldx     #$21

lf97e   dex
        beq     lf995

        bit     $911f
        bne     lf97e

lf988   bit     $911f
        beq     lf988

        lda     $95
        ror
        ror
        ora     #$40
        sta     $a3

lf995   ldx     #$02

lf997   rts

        ;       Jiffy byte in:

iecin   sei
        bit     $a3
        bvs     l7be5

        ;       orig byte in

        lda     #$00
        jmp     $ef1c

        ;       jiffy in

l7be5   lda     $911f
        and     #$03
        beq     l7be5

        lda     #$80
        sta     $9c
        txa
        pha
        pha
        pla
        pha
        pla
        lda     $912c
        and     #$dd
        sta     $912c
        ora     #$20
        tax
        bit     $9c
        bit     $9c
        bit     $9c
        lda     $911f
        ror
        ror
        sei
        and     #$80
        ora     $911f
        rol
        rol
        sta     $b3
        lda     $911f
        ror
        ror
        and     #$80
        sei
        ora     $911f
        rol
        rol
        sta     $c0
        lda     $911f
        stx     $912c
        sta     $9c
r16     jsr     lec4e
        sta     $a4
        pla
        tax
        lda     $9c
        ror
        ror
        bpl     l7c54
        bcc     lfc4f

        lda     #$42
        jmp     $eeb9

        ;       Jiffy byte out:

iecout  bit     $94
        bmi     leeed

        sec
        ror     $94
        bne     leef2

leeed   php
        pha
r17     jsr     new_iecout
        pla
        plp

leef2   sta     $95

        clc
        rts

new_iecout

lfc41   sei
        bit     $a3
        bvs     lfc59

        lda     $a3
        cmp     #$a0
        bcs     lfc59

r18     jmp     old_iecout

lfc4f   lda     #$40
        jsr     $fe6a

l7c54   lda     $a4

l7c56   clc
        rts

jiffy_out

lfc59   txa
        pha
        lda     $95
        lsr
        lsr
        lsr
        lsr
        tax
r19     lda     lfcce,x
        pha
        txa
        lsr
        lsr
        tax
r20     lda     lfcce,x
        sta     $b3
        lda     $95
        and     #$0f
        tax
        lda     #$02

l7c76   bit     $911f
        beq     l7c76

        lda     $912c
        and     #$dd
        sta     $9c
        pha
        pla
        pha
        pla
        cmp     ($00,x)
        sta     $912c
        pla
        ora     $9c
        sei
        sta     $912c
        lda     $b3
        ora     $9c
        ora     $9c
        sta     $912c
r21     lda     lfbba,x
        ora     $9c
        sei
        sta     $912c
r22     lda     lf39e,x
        ora     $9c
        sei
        sta     $912c
        sei
        and     #$dd
        bit     $a3
        bmi     l7cb7

        ora     #$02

l7cb7   sta     $912c
        pla
        tax
        sei
        lda     $9c
        ora     #$02
        sta     $912c
        lda     $911f
        and     #$02
        beq     l7c56

        ;       timeout

        jmp     $eeb7

        ;       one byte out of two nybbles

lec4e   lda     #$0f
        and     $b3
        sta     $b3
        lda     $c0
        asl
        asl
        asl
        asl
        ora     $b3
        rts

disk_close_lo

r23     jsr     untalk

        lda     #$e0
        byte    $2c

disk_close_sa

        lda     #$e1

r24     jsr     disk_listen_2

        ;       Jiffy untalk/unlisten:

unlisten

        lda     #$3f

r25
unltk   jsr     lee1c
        jsr     $eec5

        txa
        ldx     #$0b

lef0f   dex
        bne     lef0f

        tax

        jsr     $ef84
        jmp     $e4a0

r26
untalk  jsr     ora80

        lda     #$5f
        bne     unltk

disk_talk

        pha

        lda     #$00
        sta     $90

        lda     $ba
r27     jsr     talk

        pla

        ;       Jiffy talk SA:

talksa

        sta     $95
r28     jsr     lee40
        jmp     $eed3

        ;       Jiffy listen SA:

listensa

        sta     $95
r29     jsr     lee40
        jmp     $eec5

        ;       Jiffy data table:

lfcce   byte    $00,$02,$20,$22,$00,$02,$20,$22,$00,$02,$20,$22,$00,$02,$20,$22
lfbba   byte    $00,$00,$20,$20,$00,$00,$20,$20,$02,$02,$22,$22,$02,$02,$22,$22
lf39e   byte    $00,$20,$00,$20,$02,$22,$02,$22,$00,$20,$00,$20,$02,$22,$02,$22

        ;       ----------------------------------------
        ;       SYS procs LOAD
        ;       ----------------------------------------

        ;       file not found error

mold3   jmp     $f787

        ;       normal load

mold2   jmp     $f56d

        ;       old load proc

mold1   jmp     $f549

        ;       SJ Load vector:

mload   sta     $93

        ;       only for IEC

        ldx     $ba
ncpk2   cpx     #$08
        bcc     mold1

        ;       checking for Jiffy Kernal

        ldy     #$4a
        cpy     $e438
        beq     mold1

        lda     #$00
        sta     $b8

        ;       channel

        lda     #$f0
r30     jsr     diecnm
        bcs     mrts

        ;       print "searching for"

        jsr     $f647

        ;       filename (directory)

        ldy     #$00
        lda     ($bb),y
        cmp     #$24
        beq     mold2

        lda     #$60
r31     jsr     disk_talk

        ;       load address lo

r32     jsr     iecin
        tax

        lda     $90
        lsr
        lsr
        bcs     mold3

        ;       load address hi

r33     jsr     iecin

        cpy     $b9
        bne     madr

        ldx     $c3
        lda     $c4

madr    stx     $ae
        sta     $af

        ;       print "loading/verifying"

        jsr     $f66a

        ldx     $b9

        ;       Jiffy or no Jiffy

        bit     $a3
        bvs     mfb1f

        ;       normal load

        jsr     $f58a
        bcs     err

r34
myloe   jsr     disk_close_lo

        ldx     $ae
        ldy     $af

        clc
mrts    rts

        ;       Jiffy fastload init:

r35
mfb1f   jsr     untalk
        lda     #$61
r36     jsr     disk_talk

        ;       Jiffy fastload start:

        sei
        lda     $b2
        ldy     #$00
        pha

        ;       read STOP key

mfb25   jsr     $f755
        cmp     #$fe
        beq     mfb5b

        lda     $912c
        and     #$dd
        tax
        ora     #$20
        sta     $b2
        stx     $912c
        lda     #$80
        sta     $9c

mfb3d   lda     $911f
        lsr
        bcc     mfb3d

        and     #$01
        beq     mfb67

mfb47   ldx     #$6d

mfb49   bit     $911f
        beq     mfb54

        dex
        bne     mfb49

        lda     #$42
        byte    $2c

mfb54   lda     #$40
        jsr     $fe6a

        clc
        byte    $24

        ;       stop

mfb5b   sec

        pla
        sta     $b2
        bcc     myloe

err     jmp     $f6cb

mfb67   lda     #$02

mfb69   bit     $911f
        beq     mfb69

mfb6e   pha
        pla
        sei
        lda     $b2
        sta     $912c
        lda     #$01
        bit     $911f
        beq     mfb25

        stx     $912c
        lda     $911f
        ror
        ror
        sei
        and     #$80
        ora     $911f
        rol
        rol
        sei
        sta     $b3
        lda     $911f
        ror
        ror
        and     $9c
        ora     $911f
        rol
        rol
        sta     $c0

        lda     $af
        bmi     m8090

r37
mfb9e   jsr     lec4e

        cpy     $93
        bne     mfbbs

        sta     ($ae),y

mfba7   inc     $ae
        bne     mfb6e

        inc     $af
        bcs     mfb6e

        ;       do not overwrite the memory at $8000-$9fff

m8090   cmp     #$a0
        bcs     mfb9e

mfbbs   sec
        bcs     mfba7

        ;       ----------------------------------------
        ;       SYS procs VERIFY
        ;       ----------------------------------------

comm    lda     #$0f
        sta     $b9

open    jsr     $ffc0

        ldx     $90
        rts

serif   cmp     #$40
        bcc     xerif

        ;       send command

r38     jsr     comm

xerif   jsr     $ffe7

        ;       release IEC bus (on VIC-20)
        ;       (POKE 37137,127)

        lda     #$7f
        sta     $9111

        ;       (jump to Basic after VERIFY)

        ldx     #$fb
        txs

        cli

        jmp     ($c002)

perif   plp
        rts

verif   ldx     $9d
        beq     perif

        ldx     $ba
        inc     $b8
        cpx     #$04
        ldx     $b7
        bcc     perif
        bne     derif

        ;       read error channel

r39
nerif   jsr     comm
        bne     xerif

        inx
        jsr     $ffc6

        lda     #$0d
        jsr     $ffd2

cerp    jsr     $ffcf
        bcs     cerif

        jsr     $ffd2

        lda     $90
        beq     cerp
        bne     cerif

derif   cmp     #$24
        bne     serif

        ;       print directory

r40     jsr     open
        bne     xerif

        inx
        jsr     $ffc6

        jsr     $ffcf
        jsr     $ffcf

vdir    lda     #$0d
        jsr     $ffd2

        jsr     $ffe1
        beq     cerif

        jsr     $ffcf
        jsr     $ffcf

        lda     $90
        bcs     cerif
        bne     cerif

        jsr     $ffcf
        sta     $14
        tax
        jsr     $ffcf
        pha

        jsr     $ddcd

        pla
        ora     $14
        bne     vlin

        lda     #$20
        jsr     $ffd2
        jsr     $ffd2

vlin    jsr     $ffcf
        tay
        bcs     cerif
        beq     vdir

        ldy     $d3
        cpy     #$15
        bcs     vlin

        cpy     #$03
        bne     vtov

        cmp     #$20
        beq     vlin

vtov    jsr     $ffd2
        bcc     vlin

cerif   lda     #$01
        jsr     $ffc3

r41     jmp     xerif

        ;       "?*" (for first file if no filename)

fname   byte    $3f,$2a

        ;       ----------------------------------------
        ;       Jiffy I/O code (from SJ128)
        ;       ----------------------------------------

        ;       Jiffy CLRCH/CLALL ($0322/$032c):

jclall  lda     #$00
        sta     $98

jclrch  ldx     #$03
        cpx     $9a
        bcs     jc1

r42     jsr     unlisten

jc1     cpx     $99
        bcs     jc2

r43     jsr     untalk

        ;       standard clall

jc2     jmp     $f403

        ;       Jiffy GETIN ($032a):

jgetin  lda     $99
        cmp     #$08
        bcs     jg1

        ;       standard getin

        jmp     $f1f5

        ;       Jiffy BSOUT ($0326):

jbsout  pha
        lda     $9a
        cmp     #$08
        bcs     jb2

        ;       standard bsout

        jmp     $f27b

jb2     pla
r44     jmp     iecout

verf    sty     $b9
r45     jmp     verif

        ;       Jiffy BASIN ($0324):

jbasin  lda     $99
        cmp     #$08
        bcc     jg2

jg1     lda     $90
        bne     jg3

r46     jmp     iecin

        ;       standard basin

jg2     jmp     $f20e

        ;       standard iecin

jg3     jmp     $f268

        ;       Jiffy CKOUT ($0320):

r47
jckout  jsr     jchkcm
        bcc     jo2

        tax
r48     jsr     listen

        lda     $b9
        bpl     jo3

        jmp     $f33a

        ;       standard ckout

jo2     jmp     $f314

r49
jo3     jsr     listensa
        jmp     $f342

        ;       Jiffy CHKIN ($031e):

r50
jchkin  jsr     jchkcm
        bcc     ji2

        tax
r51     jsr     talk

        lda     $b9
        bpl     ji3

        jmp     $f2f8

        ;       standard chkin

ji2     jmp     $f2d2

r52
ji3     jsr     talksa
        jmp     $f301

jchkcm  jsr     $f3cf
        beq     ji1

        ;       file not open error

        pla
        pla
        jmp     $f784

ji1     jsr     $f3df

        lda     $ba
        cmp     #$08

        rts

        ;       ----------------------------------------
        ;       SAVE and LOAD routines by SDOS
        ;       ----------------------------------------

        ;       SAVE with memory control:

r53
mysave  jsr     xcontr
        bne     origs

        ;       if it fails, the Kernal takes over the task

r54     jsr     msave
        bcc     mro

        lda     $90
        and     #$bf
        beq     mro

        ;       (*+1 = $1949) <- (*+1 = $0949) <- original SAVE vector

origs   jmp     $f685

        ;       LOAD with memory control:

myload  sta     $93
r55     jsr     xcontr

        php
        lda     ($bb),y

        ldx     $93
        sty     $b8
        bne     verf

        ldx     $b7
        bne     mylo

        ;       "?*" (for first file if no filename)

        byte    $a2
r56     byte    <fname
        lda     #>fname
        stx     $bb
        sta     $bc
        ldx     #$02
        stx     $b7
        bne     nylo

mylo    cmp     #$24
        bne     pylo

nylo    sty     $b9

pylo    lda     $9d
        plp
        bne     mlo

        ;       if it fails, the Kernal takes over the task

        lda     $93
r57     jsr     mload
        bcc     mro

        lda     $90
        and     #$bf
        beq     mro

        ldy     #$00
        lda     $9d
        sty     $9d

mlo     pha
        lda     $93

        ;       (*+1 = $198d) <- (*+1 = $098d) <- original LOAD vector

origl   jsr     $f549

        sta     $14
        pla
        sta     $9d
        lda     $14

mro     pha

        ;       release IEC bus (on VIC-20)
        ;       (POKE 37137,127)

        lda     #$7f
        sta     $9111

        bcs     mso

        txa
        pha
        tya
        pha

r58     jsr     msontr

        pla
        tay
        pla
        tax

mso     pla
        rts

        ;       ----------------------------------------
        ;       Memory controlling subroutine
        ;       ----------------------------------------

        ;       (disables itself if the memory occupied by SDOS gets corrupted)

mno     lda     #$60
r59     sta     mcntr

        ldy     #$1f
mnlp    lda     $fd6d,y
        sta     $0314,y
        dey
        bpl     mnlp

        ;lda     $c4
        ;pha
        ;lda     $c3
        ;pha
        ;jsr     $ff8a
        ;pla
        ;sta     $c3
        ;pla
        ;sta     $c4

        ;       device number control

xcontr  lda     $ba
        cmp     #$08
        beq     xmo
        cmp     #$02
        bcs     xro

        and     $b9
        sta     $b9

        ;       last accessed device

xmo     lda     $a8

xro     and     #$0f
        bne     xno

        lda     #$08

xno     sta     $a8
        sta     $ba

        ;       memory control

mcontr  clc
msontr  ldy     #$00

        ;       checking for Jiffy Kernal

        lda     #$4a

        ;       (*+1 = $19db) <- (*+1 = $09db)

mcntr   eor     $e438
        beq     mno

        ;       (*+1 = $19e0) <- (*+1 = $09e0) <- address

maddr   lda     #$04

        ;       (*+1 = $19e2) <- (*+1 = $09e2) <- size

msize   ldx     #$06

        sty     $14
        sta     $15

mclp    eor     ($14),y
        iny
        bne     mclp

        inc     $15
        dex
        bne     mclp

        tax
        beq     myo
        bcc     mno

        dec     $14
        dec     $15

        sta     ($14),y

myo     sty     $90
        rts

        ;       CRC byte placeholder (must be zero here)

crc     byte    $00

        ;       (* = $1a00) <- (* = $0a00) <- relocation table (only low bytes)

rtbl    byte    $04

        byte    $0f,$11,$13,$15,$17,$19,$1b,$1d,$1f
        byte    $21,$23

        byte    <r00+2,<r01+2,<r02+2,<r03+2,<r04+2,<r05+2,<r06+2,<r07+2
        byte    <r08+2,<r09+2,<r10+2,<r11+2,<r12+2,<r13+2,<r14+2,<r15+2
        byte    <r16+2,<r17+2,<r18+2,<r19+2,<r20+2,<r21+2,<r22+2,<r23+2
        byte    <r24+2,<r25+2,<r26+2,<r27+2,<r28+2,<r29+2,<r30+2,<r31+2
        byte    <r32+2,<r33+2,<r34+2,<r35+2,<r36+2,<r37+2,<r38+2,<r39+2
        byte    <r40+2,<r41+2,<r42+2,<r43+2,<r44+2,<r45+2,<r46+2,<r47+2
        byte    <r48+2,<r49+2,<r50+2,<r51+2,<r52+2,<r53+2,<r54+2,<r55+2
        byte    <r56+2,<r57+2,<r58+2,<r59+2

        byte    <maddr+1

rtbh    byte    $1a,$1e,$29,$31,$3c

        ;       ----------------------------------------
        ;       SETUP subroutines
        ;       ----------------------------------------

setup   ldx     #$80

        sty     $b8
        sty     $b9
        stx     $9d

        ;       checking for * or = in filename (to load another file if so)
        ;       (including some self-modification of the startup code)

        lda     #$10
        ldx     #$0a
        sta     $0156
        stx     $0157

silp    dec     $b7
        beq     lasc

        lda     ($bb),y

        inc     $bb
        bne     silc

        inc     $bc

silc    cmp     #$2a
        beq     sasc
        cmp     #$3d
        bne     silp

sasc    inc     $b7

        ;       last accessed device

lasc    ldy     $ba
        bne     haba

        ldy     #$08
        sty     $ba

haba    sty     $a8

        ;       original jumping vectors

        ldx     $0330
        lda     $0331
        stx     origl+$1001
        sta     origl+$1002

        ldx     $0332
        lda     $0333
        stx     origs+$1001
        sta     origs+$1002

        ldx     #$58
hlp     lda     $68,x
        sta     $1380,x
        dex
        bpl     hlp

        cli

        ;       checking for C= or CTRL pressed

        cpy     #$04
        jsr     keycr+$1000
        bcc     kernal

        ;       if any pressed, skip drive check
        ;       (either C= or Ctrl alone -> force SJLOAD, both -> Kernal LOAD)

        bne     xomm

        ;       "ui" (soft reset for drive identification)

        lda     #$0f
        stx     $90
        inc     $b8
        sta     $b9

        lda     #$02
        ldx     #$0c
        ldy     #$14
        jsr     $ffbd

        jsr     $ffc0

        ldx     $90
        bne     xomm

        stx     $a2
tlp     lda     $a2
        cmp     #$10
        bcc     tlp

        inx
        jsr     $ffc6

comp    jsr     $ffcf
cpc     sta     $1300

        lda     $90
        bne     xomm

        inc     cpc+$1001
        bpl     comp

xomm    jsr     $ffe7

xlp     ldx     #$58
zlp     lda     $1380,x
        sta     $68,x
        dex
        bpl     zlp

flp1    inx
        cpx     #$04
        bcs     nock

        ldy     cpc+$1001

flp2    dey
        bmi     flp1

        lda     stab1+$1000,x
        cmp     $1300,y
        bne     flp2
        lda     stab2+$1000,x
        cmp     $1301,y
        bne     flp2

        cpx     #$02
        ldy     #$41
        bcc     kernal

        ;       checking for NTSC Kernal (disable SJ for 1541/71 in this case)

        cpy     $e475
        bne     nock

        ;       disable I/O routines and use Kernal for LOAD and SAVE
        ;       (SDOS only provides wedge functionality in this case)

kernal  ldx     #$11
klp     lda     $031e,x
        sta     wtb+$1000,x
        dex
        bpl     klp

        stx     ncpk1+$1001
        stx     ncpk2+$1001
        inc     mcntr+$1001

        ;       checking for C= or CTRL pressed

nock    ldy     #$4a
        jsr     keycr+$1000

        ;       checking for Jiffy Kernal (disable SJ in this case)

nix     cpy     $e438
        beq     neor

        cmp     #$06
neor    inx
        bcc     nick

        ;       if Ctrl+C= keys both pressed: force Kernal LOAD

        clc
        bne     kernal

        ;       looking for a relocation target

nick    tax
        lda     $38
        sbc     #$06
        cmp     #$20
        bcs     xick
        lda     #$00

        ;       if Ctrl+C= keys: trying locations at $a0/$ba/$0a/$04/$9a/$79
        ;       if Ctrl key pressed: trying locations at $0a/$04/$9a/$79 only
        ;       if C= key pressed: trying locations at $9a/$79 only

xick    cpx     #$00
        sta     relo+$1000
        bne     xelp

        ;       otherwise: trying locations at $ba/$a0/$ba/$0a/$04/$9a/$79
        ;       (if none of them succeeds: trying end of Basic area instead)

        ldx     #$07

xelp    ldy     relo+$1000,x
        beq     xeyo

        sty     $17

        sta     ($16),y
        cmp     ($16),y
        bne     xeno

        tya
        sta     ($16),y
        cmp     ($16),y
        beq     xelo

xeno    dex
        bpl     xelp

xeyo    ldy     #$14
        sty     $17

xelo    lda     #$10

        ;       $02bc/$02e1 <- sys700/sys737 for uninstall/reinstall

        sty     $02e3

        ;       if both Basic and the install target space are within the same
        ;       lowest block, Basic will be moved up to $1001 (to not collide)
        ;       (screen is also moved if needed)

        cmp     $17
        bcc     nack
        cmp     $2c
        bcc     nack

        ldx     #$1e
        sta     $2c
        stx     $38

        stx     $0288
        jsr     $e518

nack    lda     #$00
        ldy     #$19
        ldx     #$48
        sta     $14
        sty     $15
        sta     $16

        ;       $02bc/$02e1 <- sys700/sys737 for uninstall/reinstall

        sta     $02e2

        ;       relocation of the addresses in main code

rlp1    txa
        ldy     #$04

rlp2    cmp     rtbh+$1000,y
        beq     rtba
        dey
        bpl     rlp2

        byte    $2c
rtba    dec     $15

        ldy     rtbl+$1000,x
        lda     ($14),y
        cmp     #$0a
        bcs     rex

        sbc     #$03
        clc
        adc     $17
        sta     ($14),y

rex     dex
        bpl     rlp1

        ;       $02bc/$02e1 <- sys700/sys737 for uninstall/reinstall

        lda     #$8a
        sta     $02bd
        stx     $02be

        lda     #$4c
        ldy     #$00
        ldx     #$06
        sta     $02bc
        sta     $02e1

        ;       relocation of the main code

rlp3    lda     ($14),y
        sta     ($16),y
        iny
        bne     rlp3

        inc     $15
        inc     $17
        dex
        bne     rlp3

        sec
        jsr     msontr+$1000
        jsr     $02e1

        tya
        stx     $2b
        sta     ($2b),y
        inc     $2b

        ldy     #>mess+$10
        stx     $c6
        jsr     $cb1e

        ;       checking for C= or CTRL pressed

keycr   lda     $0173
        ora     $028d
        sta     $0173
        and     #$06
        rts

*=$0bf3

relo    byte    $00,$79,$9a,$04,$0a,$ba,$a0,$ba

        ;       em,vi,54,57 (emulation, vice/virtual, 1541, 1571)

stab1   byte    $45,$56,$35,$35
stab2   byte    $4d,$49,$34,$37

        ;       "sdos v1.3"

mess    byte    $93,$0e,$53,$44,$4f,$53,$20,$56,$31,$2e,$33,$0d,$00

        ;       (* = $0c10)

        ;       compiled with CBM prg Studio v3.13 (by Arthur Jordison)
        ;       http://ajordison.co.uk/

        ;       Rosetta Interactive Fiction project homepage:

        ;       http://istennyila.hu/rosetta

        ;       v0.13 (c) 2012-2023 by Robert Olessak

        ;       SDOS project page:

        ;       http://istennyila.hu/sdos
