
        ;       SV64 (legacy) - used to be called as "!"

        ;       SDOS (v1.3) - C64 floppy speeder and disk utility
        ;       1986/2008/2016-2023 by Carroll/1570/Olessak
        ;       Public Domain (open-source and freeware)

        ;       VDOS is written by Edward Carroll (in 1986);
        ;       disassembled and modified for Jiffy protocol by 1570 (in 2008);
        ;       both sources are re-assembled by Robert Olessak (in 2016-2023).

        ;       SDOS is based on SJLOAD and VDOS. This is a common boot loader
        ;       that checks the drive and chooses among three possibilities:

        ;       1.) if JiffyDOS protocol is supported, it starts as SJLOAD;
        ;       2.) if not, it tries to start as VDOS (which is possible on
        ;           1541 and 1571 drives in C64 mode);
        ;       3.) if none of them is present, it stays at Kernal load.

        ;       SJ64 is a subset of SDOS: SJLOAD only (with VDOS code removed).
        ;       SV64 is a subset of SDOS: VDOS only (with SJLOAD code removed).

        ;       Starting (in all cases):

        ;       LOAD"!*PROGRAM",8(,1) - (auto)start SDOS, fastload program
        ;       LOAD"!",8(,1) / RUN - (auto)start SDOS only

        ;       (on 1581 the * does not work, so please type = instead)

        ;       Usage (after having been installed):

        ;       VERIFY - send commands, read floppy status
        ;       VERIFY"$" - display directory (only on the screen)

        ;       LOAD"$" - load directory from last accessed device
        ;       LOAD"PROGRAM" - load program from last accessed device
        ;       LOAD"PROGRAM"(,8),1 - as above (as a machine code)
        ;       LOAD"PROGRAM"(,8),0 - as above (as a Basic code)

        ;       LOAD (without filename) => LOAD"?*",8(,1) (load first file)

        ;       The loader uses $02a7-$02ff and $ca00-$cfca as framework area
        ;       when active and $fa00-$ffca when hiding (under Kernal ROM).

        ;       New features (of SDOS in v1.0 to v1.2):

        ;       if using LOAD"!*PROGRAM",8(,1) the program will also be autorun
        ;       holding SHIFT key skips the autorun
        ;       holding C= key forces always VDOS (and skips drive check)
        ;       holding CTRL key forces always SJLOAD (and skips drive check)
        ;       holding both C= and CTRL keys at once forces always Kernal load
        ;       screen and IRQ are disabled while loading (in all three modes)
        ;       typing 0 or 1 for device number means latest with ,0 or ,1
        ;       typing no device number at all means latest once again
        ;       number 8 is substitute for last accessed device
        ;       specifying no filename means load first file
        ;       (plus several further minor bugfixes)

        ;       New features (of SDOS in v1.3):

        ;       no limit in loadable file size any more
        ;       loading even possible to RAM under the I/O
        ;       directory will also be fastloaded if possible
        ;       better optimization, device number, error handling etc.
        ;       (plus several further minor/major bugfixes)

        ;       at any time it can be re-installed by SYS737
        ;       at any time it can be un-installed by SYS700

        ;       memory swapping can be disabled by SYS727

        ;       (Normally the code resides in the "hidden" RAM under the Kernal
        ;       area, and is brought forth to $ca00 only during loading, then
        ;       afterwards swapped there back. So remains the $ca00-$cfca area
        ;       untouched and stays usable for any other purposes. You may
        ;       switch this feature off if you do not need it - to make things
        ;       faster, or to also use the "hidden" RAM.)

*=$0326

        ;       CHROUT and STOP vector <= Basic chaining + line number

        word    setup,$f6ed

        ;       63213 syspeek(44)*256+1119

        byte    $9e,$c2,$28,$34,$34,$29,$ac,$32,$35,$36,$aa,$31,$31,$31,$39,$00
        byte    $00,$00

*=$033c

        ;       (* = $033c) <- (* = $0817) <- autostart (sys828)

        ;       ----------------------------------------
        ;       SETUP init (entry point for autostart)
        ;       ----------------------------------------

setup   jsr     $ff8a

retup   jsr     reset

        ldx     #$06

stp     lda     lca00,y
        sta     $ca00,y
        iny
        bne     stp

        inc     stp+2
        inc     stp+5
        dex
        bne     stp

        jsr     drive+$c600
        jsr     $02ce

        jsr     $a644

        dec     $b7
        bmi     basc

        ldx     $2b
        ldy     $2c

        jsr     $ffd5
        bcs     basc

        stx     $2d
        sty     $2e

        jsr     $a533

atp     lda     #$00
        ldx     #$05
        ora     $028d
        and     #$01
        bne     basc

        stx     $c6

rtp     lda     run-1,x
        dex
        sta     $0277,x
        bne     rtp

        ;       basic warm start

basc    jmp     ($a002)

*=$038c

        ;       "run:" (into keyboard buffer)

run     byte    $52,$55,$4e,$3a,$0d

*=$0391

        ;       drive reset and installing helper

reset   ldy     #$09

        ;       (patching for SYS828)

lchrp   lda     pce03-1,y
        dey
        sta     vdstr,y
        bne     lchrp

        lda     #$80

        sty     $90
        sta     $9d
        sty     $b9
        sty     $c6

        ;       checking for * or = in filename (to load another file if so)

silp    dec     $b7
        beq     lasc

        lda     ($bb),y

        inc     $bb
        bne     silc

        inc     $bc

silc    cmp     #$2a
        beq     sasc
        cmp     #$3d
        bne     silp

sasc    inc     $b7

        ;       checking for modified LOAD vector

lasc    lda     $0330
        ldx     $0331
        bpl     crchk

        sta     lcf8b+1
        stx     lcf8d+1

        sta     s02c4+1
        stx     s02c4+2

        ;       checking for DTV

        cpx     #$f7
        bne     crchk

        ;       (self-modification code for DTV)

        ldx     #$19
lchlp   lda     tce0a-1,x
        dex
        sta     lce09+1,x
        bne     lchlp

        ;       turn off all fast modes (and check) <- (required for timing)

crchk   jsr     lce09
        jsr     lcf12

        ;       ($09a3 = lcfa3) <- $cfa3 before copying (value of $d030)

        sta     lcfa2+1

        ;       (* = $03e8)

        ;       ----------------------------------------
        ;       SETUP subroutines
        ;       ----------------------------------------

        ;       checking for modified LOAD vector and machine type

check1  ;sei

        ;       detect DTV

        txa
        beq     crock

        ;       detect MEGA65 using POKE 0,64

        lda     #$40
        ldx     $00
        sta     $00
        cpx     $00
        stx     $00
        beq     qrock

        tya

        ;       CPU check ($1a = INC A on 4510 or 65816, but NOP on 6510 CPU)

        byte    $1a
        beq     chock

        ;       CPU check ($eb = NOP on 65C02, XBA on 65816, ROW $AA98 on 4510)

        byte    $eb
        tya
        tax

        ;       4510 (C65 or MEGA65) found -> normal load (like NTSC/DTV)

        asl

        byte    $d0
        ;bne     qrock

        ;       (* = $0400)

lca00   byte    $06

        byte    $eb

        ;       65C02 found -> Kernal LOAD (not suitable for timing)

        tay
        ;beq     chyck

        ;       65816 (SuperCPU) found -> turbo off <- (required for timing)
        ;       (not necessary here when already done above)

        ;sta     $d07a

        bne     chyck

        dex
        byte    $2c

        ;       self-modification on C65/MEGA65 (device number for Kernal LOAD)

qrock   ldx     #$0a

        stx     lccc6+1
        ;stx     c65ck+1
        ;stx     c65cx+1

        ;       $cf10 <- $ce24 (RTS) <-> $ce09 (on C65/MEGA65 or DTV)

crock   ldy     #$09
        sty     lcf0f+1

chock   ;iny
chyck   ;sty     $aa

        ;       last accessed device

        lda     $ba
        bne     haba

        lda     #$08
        sta     $ba

haba    ldx     #$58

        sta     $a8
        sta     $a9

hlp     lda     $68,x
hlx     sta     $0a60,x
        lda     s02a7,x
        sta     $02a7,x
        dex
        bpl     hlp

        stx     $90

        ;       checking for C= or CTRL depressed

check2  ;jsr     $ff9f

        lda     $028d

;        ldx     $ba
        ora     atp+1

;c65ck   cpx     #$04
        sta     atp+1
;        bcs     orck

;c65cx   ldx     #$08

orck    jmp     rwap

        and     #$06
        cmp     #$06
        bcc     hlx+1

        ;       self-modification (revert to Kernal LOAD)

        pla
        pla

        ;       self-modification (from VDOS to KERNAL) <- LDY #$FF

kerl    ldy     #$ff

        ;rts
        ;byte    $ff

        ;       (always branch on BCC if Kernal LOAD)

        sty     $ccc7

        ;bmi     swap

        ;       self-modification <- if no swapping

        ;       (and modify controlling data pointer from $ca/$fa to $cc/$fc
        ;       except on VDOS where it must stay $ca as can be seen here
        ;       since VDOS needs its routines at the $ca00-$cbff area)

        ldy     #$cc
        sty     swecc+$c601

        bmi     swap

        ;       drive check

drive   lda     #$2c
        ;lda     #$a0
        ldy     #$ca

        sta     orck+$c600
        ;sta     kerl+$c600

        sty     swc+$c601
        sty     swync+$c601

        ldy     #$fa
        lda     #$cf
        sty     swd+$c601
        sta     swynd+$c601

        ;       prompt with version number and
        ;       make CRC (of $02b0-$02ff + $ca6f-$cabe + $cfcb-$cff5 at first)

        jsr     $cd30

        ;       checking for C= or CTRL depressed

        jsr     check2+$c600

swap    jmp     xwap+$c600

*=$046f

        ;       (* = $ca6f)

        ;       ----------------------------------------
        ;       VDOS ($ca6f-$cfca)
        ;       ----------------------------------------

        ;       drive code (in drive at $0600-$0699) of VDOS

l600    ldy     #$00

l602    tya
        and     #$0f
        tax
        lda     $0689,x
        sta     $0400,y
        tya
        lsr
        lsr
        lsr
        lsr
        tax
        lda     $0689,x
        sta     $0500,y
        iny
        bne     l602

l61b    lda     $18
        sta     $06
        lda     $19
        sta     $07

l623    lda     #$b0
        sta     $00

l627    lda     $00
        bmi     l627

l62b    lda     #$80
        sta     $00

l62f    lda     $00
        bmi     l62f

l633    sei

        cmp     #$01
        beq     l63d

l638    lda     #$ff
        sta     $0300

l63d    lda     #$02
        sta     $1800
        asl

l643    bit     $1800
        beq     l643

l648    ldx     $0300,y

l64b    bit     $1800
        bne     l64b

l650    lda     $0500,x
        sta     $1800
        asl
        and     #$0f
        sei
        sta     $1800
        lda     $0400,x
        sei
        sta     $1800
        asl
        and     #$0f
        sei
        sta     $1800
        lda     #$04
        iny
        sei
        sta     $1800
        bne     l648

l674    cli

        lda     $0301
        sta     $07
        lda     $0300
        beq     l699

l67f    bmi     l699

l681    cmp     $06
        sta     $06
        beq     l62b

l687    bne     l623

l689    byte    $0f,$07,$0d,$05,$0b,$03,$09,$01,$0e,$06,$0c,$04,$0a,$02,$08,$00

l699    rts
        
        ;       VDOS code (start loading) <- called as subroutine

vload   ;ldx     #$00
        ;ldy     #$9b

        ;       sending drive code to the drive
        ;       5 x $1f (31) = $9b (155), although we only need $9a (154) here

lvlp1   cpx     #$1e
        lda     l600+$c5ff,y
        sta     vdstr+$c5e1,x
        dey
        inx
        bcc     lvlp1

        ;       drive command (perform "M-W" + $06xx + byte length) <- write

        lda     #$57
        sty     vdstr+$c602
        jsr     vdcmd+$c600

        tya
        bne     lvlp1

        ;       drive command (perform "M-E" + $0600) <- execute

        ldx     #$20
        lda     #$45
        stx     $aa

        ;       (perform drive command)

vdcmd   sta     vdstr+$c603

        ;       (listen)

        jsr     $cf1c

        ldx     #$25
lvlp2   lda     vdstr+$c5e0,x
        dex
        jsr     $ffa8
        cpx     $aa
        bne     lvlp2

        ;       (unlisten)

        jmp     $ffae

*=$053c

        ;       (* = $cb3c)

        ;       ----------------------------------------
        ;       common helper ($cb3c-$cb63)
        ;       ----------------------------------------

        ;       display message, install load vector etc.

sweax   ldy     #<$02e3
        ldx     #>$02e3

        sty     $0330
        stx     $0331

        ;       re-make CRC

        jsr     $cfb1

        ;       make CRC (of $02b0-$02ff + $ca6f-$cabe + $cfcb-$cff5 at first)
        ;       (or $ffcb-$fff5 later)

        bit     $02ea
        sta     $cf67
        bmi     sweay

        ;       self-modification (for bad CRC of swap r.) <- if no swapping

swecc   lda     #$ca
        inx

        ;       (and modify controlling data pointer from $ca/$fa to $cc/$fc
        ;       except on VDOS where it must stay $ca as can be seen here
        ;       since VDOS needs its routines at the $ca00-$cbff area)

        sta     $ced7
        eor     #$30
        sta     $cefb

sweay   ldy     #$37
        stx     $cf76
        sty     $01

        rts

*=$0564

        ;       (* = $cb64)

        ;       ----------------------------------------
        ;       common LOAD/VERIFY routines ($cb64-$ccff)
        ;       ----------------------------------------

        ;       print directory

lcb64   jsr     $cc8e
        jsr     $cbb2

lcb6a   stx     $a7
        jsr     $aad7

        jsr     $cbaf

        jsr     $bdcd
        jsr     $ab3f

lcb78   jsr     $cbc0

        tax
        beq     lcb6a

        cmp     #$22
        bne     lcb84

        sta     $a7

lcb84   ldy     $d3
        lda     #$3a
        cpy     #$1c
        beq     lcb9c
        bcs     lcb78

        cpy     #$18
        bcc     lcba9
        bne     lcba5

        lda     #$2c
        cpx     #$20
        beq     lcb9c

        cpx     #$40
lcb9c   ldy     $a7
        beq     lcb78
        bcc     lcba9

        jsr     $ffd2

lcba5   cpx     #$3c
        bcc     lcb78

lcba9   txa
        jsr     $ffd2
        bcc     lcb78

        ;       get serial byte (with error handling)

lcbaf   jsr     $cbb2

lcbb2   bit     $91
        bmi     lcbbc
        bpl     lcbf6

        ;       "run:" (on screen at ready prompt) <- $05b8 = 1464

lcbb8   byte    $12,$15,$0e,$3a

lcbbc   jsr     $cbc0
        tax

lcbc0   lda     $90
        bne     lcbf6

        jmp     $ffa5

        ;       print directory

lcbc7   bcs     lcb64

        ;       send drive command

lcbc9   jsr     $cc1a

lcbcc   lda     ($bb),y
        jsr     $ffa8
        iny
        cpy     $b7
        bcc     lcbcc

        jsr     $ffae

        clc
        bcc     lcbfc

        ;       read error channel

lcbdc   lda     $ba
        jsr     $ffb4
        lda     #$6f
        jsr     $ff96

        jsr     $cc1d

        jsr     $aad7

lcbec   jsr     $cbc0
        jsr     $ffd2

        cmp     #$20
        bcs     lcbec

lcbf6   jsr     $cf26

        ;       (jump to Basic after VERIFY)

lcbf9   clc
        byte    $24
lcbfb   sec

lcbfc   jsr     $cf40

        ldx     #$fb
        jmp     $02a7

        ;       print directory

lcc04   bcs     lcbc7

        ;       read error channel

lcc06   bcs     lcbdc

        ;       send drive command

lcc08   and     #$c0
        bcc     lcbfb
        beq     lcbfc
        bcs     lcbc9

        ;       (* = $cc10) <- turn on fast mode

lcc10   ldy     #$01
        jsr     $cf0f
        sty     $d07b
        dey

lcc19   rts

        ;       (* = $cc1a) <- listen + error handling

lcc1a   jsr     $cf1c

        ;       (* = $cc1d) <- error handling

lcc1d   bit     $90
        bpl     lcc19

lcc21   pla
        pla

        ;       device not present error

lcc23   jsr     $cf40
        jmp     $f707

        ;       load/verify

lcc29   tax
        ;sta     $93

        lda     $ba

        cmp     #$08
        beq     lcc36
        cmp     #$02
        bcs     lcc38

        ;and     $b9
        sta     $b9

        ;       last accessed device

lcc36   lda     $a8

lcc38   and     #$0f
        bne     lcc3e

        lda     #$08

lcc3e   ldy     #$00
        cmp     #$04

        sty     $90
        ;ldx     $93
        sta     $a8
        sta     $ba

        ;       turn off fast mode + save registers

        jsr     $ceff
        sta     $cfa3

        ;       (checking if "$" and/or LOAD/VERIFY with or without filename)

        lda     $b7
        sty     $d015
        bne     lcc67

        lda     #<$02ae
        sta     $bb
        lda     #$02
        sta     $bc
lcc5d   bcs     lcc62

        lsr
        inc     $bb

lcc62   sta     $b7

        txa
        bne     lcc06

lcc67   lda     ($bb),y

        eor     #$24
        bne     lcc72

        ;       (checking if "$" and/or LOAD/VERIFY with or without filename)

        txa
        sty     $b9
        bne     lcc04

lcc72   dex
        bpl     lcc08

        ;       turn off screen

        ldx     #$0b
        stx     $d011
        sty     $d020

        ldx     $ba

        ;       (*+1 = $cc80)

lcc7f   cpx     #$08

        ldx     $b9
        sty     $a7
        sta     $aa
        stx     $ab

        ;       (* = $cc89)

lcc89   bcs     lsnc0

        ;       Kernal LOAD (orig)

lcc8b   jmp     $02bf

        ;       (* = $cc8e) <-> $ccef (see it there!)

lcc8e   jsr     $ccef

        bit     $90
        bmi     lcc21

        rts

        ;       raster synchronization (having the screen turned off) #1

lsnc0   bit     $d011
        bpl     lsnc0

;lsnc1   bit     $d011
;        bmi     lsnc1

        ;       "searching for..."

        jsr     $f5af

        ;       open file (with error handling) + talk
        ;       (if fails: device not present error)

lcc9e   jsr     $cc8e

        ;       start address (lb, hb)

        jsr     $ffa5
        tay

        ;       (if fails: file not found error)

        lda     #$bf
        and     $90
        bne     lccd7

        jsr     $ffa5
        ;sei

        ldx     $ab
        bne     lccb6

        ldy     $c3
        lda     $c4

lccb6   sty     $ae
        sta     $af

        ;sty     $ac
        ;sta     $ad

        ;       control data pointer (set forth)

        jsr     $ced4

        ;       (second try!)

        ldx     $a7
        bne     lcce5

        ;       "loading" (print message)

        jsr     $f5d2

        ;       checking for device number again, for Kernal LOAD
        ;       (on C65/MEGA65 where it must be $0a instead of $08)

        lda     $ba

        ;       (or $ff to always branch on BCC if Kernal LOAD)

        ;       (*+1 = $ccc7)

lccc6   cmp     #$08
        ldy     $aa
        stx     $aa
        bcc     lcce5
lccce   beq     lcce5

        sta     $a7

        ;       VDOS <-> SJLOAD (start loading) <- start over (or continue)

        jsr     $ce25

        ;       if speedloading failed, we try again with Kernal LOAD

        bcs     lcc9e
        ;bcc     lccef+1

        ;       file not found error

lccd7   jsr     $cf40
        jmp     $f704

        ;       Kernal LOAD (open), fetching byte by byte

lccdd   jsr     $ffa5
        jsr     $cec7
        stx     $01

lcce5   ldy     $90
        beq     lccdd

        ;       control data pointer (set back)

lcce9   jsr     $cef8

        ;       loading is ready -> jump to return from load routine

lccec   jmp     $cf30

        ;       (* = $ccef) <-> $cc8e (see it there!)

lccef   lda     #$60
        sta     $b9

        ;       (open) <- it aborts if device not present!

        jsr     $f3d5

        ;       (talk)

lccf6   lda     $ba
        jsr     $ffb4
        lda     $b9
        jmp     $ff96

*=$0700

        ;       (* = $cd00)

        ;       ----------------------------------------
        ;       SETUP subroutines
        ;       ----------------------------------------

xwap    jsr     $ffe7

xlp     ldx     #$58
zlp     lda     $0a60,x
        sta     $68,x
        dex
        bpl     zlp

        txa

        ;       release IEC bus (on C64)
        ;       (POKE 56576,199)

        ;ldy     #$c7
        ;sty     $dd00

        ;       return from load routine and/or
        ;       restore screen + fast mode (before Kernal call) etc.

        jsr     $cf40

        sta     swex+$c602

        ;       swapping $c000 and $f000 (back)

;        jsr     rwap+$c600

        ;       sys717 <- sys737 (re-install) <- sweax

;        jmp     $02ce

        ;       swapping $c000 and $f000

rwap    sei

        lda     #$34
        ldx     #>swp
        ldy     #$60

        sta     $01
        stx     s02f4+2

        sty     $fff5
        cpy     $fff5
        beq     swax

        ldx     #$df
        sty     $02ea

        bmi     xwax

*=$0730

        ;       (* = $cd30) <- 32 bytes trashed by Kernal (jsr $ff8a)

        ;       display message, install load vector, exit (once, at startup)

lcd30   ;ldx     #$00

        lda     #<$cd40
        ldy     #>$cd40

        stx     $d020
        stx     $d021

        ;       prompt with version number

        jsr     $ab1e

        jmp     sweax+$c600

        ;       "sv64 v1.3"

lcd40   byte    $99,$93,$0e,$53,$56,$36,$34,$20,$56,$31,$2e,$33,$0d,$00

*=$0750

xwax    sta     lcc5d
        stx     lceed+1
        sty     swp
        sty     rwap

swax    ldy     #$00

        jmp     s02ea

*=$0762

        ;       ----------------------------------------
        ;       patch code parts
        ;       ----------------------------------------

        ;       (* = $ce03) <- (* = $0803) <- (patching for SYS828)

pce03   byte    $1f,$06,$00,$57,$2d,$4d

        ;       (* = $ce09) <- C65/MEGA65 code (normal) <- (see above!)

pce09   lda     #$a5
        byte    $8d

*=$076b

        ;       ----------------------------------------
        ;       patch code parts
        ;       ----------------------------------------

        ;       (* = $ce0a-$ce22) <- DTV code (optional)

tce0a   byte    $01
        sta     $d03f

        byte    $32,$99

        tya
        beq     tce15

        lda     #$03

tce15   byte    $32,$00

        tya
        beq     tce1c

        lda     #$20

tce1c   sta     $d03c

        lda     #$00
        byte    $8d,$3f

*=$0784

        ;       (* = $0784) <- (* = $0c5f) <- Basic start (sys3167)

        ;       ----------------------------------------
        ;       SETUP init (entry point for Basic)
        ;       ----------------------------------------

        ;       sys3167 (= 2048 + 1119)

        sei

        ldy     #$16
        lda     #$26
        ldx     #$03

        sta     $0c
        stx     $0d

        ldx     $2c

        ;       (* = $0791) <- (* = $0c6c)

btp     lda     ($2b),y
        sta     ($0c),y
        iny
        bne     btp

        lda     $0d
        inc     $0d

        cmp     #$09
        inc     $2c
        bcc     btp

        stx     $2c

        jmp     retup
        ;jmp     setup

        ;       (* = $07a7) <- (* = $0c82)

*=$07a7

        ;       ----------------------------------------
        ;       SJLOAD/VDOS helper ($02a7-$02ff)
        ;       ----------------------------------------

        ;       (jump to Basic after VERIFY)

s02a7   jsr     $02ea

        ;       (jump to Basic after setup)

s02aa   txs

        ;       basic warm start

        jmp     ($a002)

        ;       "?*" (filename at $02ae)

s02ae   byte    $3f,$2a

        ;       (swapping forth)

s02b0   jsr     $02ea

        ldy     #$60
        cpy     $cfca
        bne     s02ea

        cli
        rts

        ;       sys700 (jsr $02bc): un-install (by restoring Kernal)

s02bc   jmp     $ff8a

        ;       (* = $02bf) <- Kernal load ($02c5-$02c6) = LOAD vector (orig)
        ;       (i.e. swapping back, calling load, then swapping forth again)

s02bf   tya
s02c0   jsr     $02ea
s02c3   sei
s02c4   jsr     $f4a5
        jsr     $02ea
        jmp     $cf40

        ;       sys717 <- sys737 (re-install)

s02cd   clc
s02ce   jsr     $02b0

        ;       re-make CRC etc.

s02d1   jsr     sweax+$c600

        clc
        bcc     s02ea

        ;       sys727 (jsr $02d7): re-install with swapping disabled
        ;       (speedloader code will stay for ever fixed at $ca00)

s02d7   jsr     $02b0

s02da   ldy     #$60
        sty     $02ea
        bpl     s02d1

        ;       sys737 (jsr $02e1): re-install

s02e1   clc
        bcc     s02ce

        ;       (NOP at $02e3 hidden in the operand!)

        jsr     $02b0
        jsr     $cc29

        ;       (swapping back)

s02ea   nop
        nop

s02ec   pha
s02ed   tya
        ldy     #$34
        sei
        pha
        sty     $01
s02f4   jsr     $ffcb
        ldy     #$37
        pla
        sty     $01
        tay
s02fd   pla
s02fe   cli
s02ff   rts

*=$0800

        ;       ----------------------------------------
        ;       common LOAD routines ($ce00-$cfca)
        ;       ----------------------------------------

        ;       (* = $ce00) <- (* = $0800) <- Basic bytes (for RUN)

lce00   byte    $00,$0a,$08

        ;       0 sys828

vdstr   byte    $00,$00,$9e,$38,$32,$38

        ;       (* = $ce09) <- C65/MEGA65 code (normal) <- (see above!)

lce09   byte    $00,$00,$00
        word    $d02f

;        ;       (* = $ce00) <- (* = $0800) <- Basic bytes (for NEW)
;
;lce00   byte    $00,$00,$00
;
;vdstr   byte    $1f,$06,$00,$57,$2d,$4d
;
;        ;       (* = $ce09) <- C65/MEGA65 code (normal) <- (see above!)
;
;lce09   lda     #$a5
;        sta     $d02f

lce0e   lda     #$96
        sta     $d02f

        lda     $d031
        and     #$bf

        dey
        bmi     lce1d

        ora     #$40

lce1d   iny
        sta     $d031

lce21   sty     $d02f

lce24   rts

        ;       (* = $ce25) <- VDOS/SJLOAD (common) <- start loading

        ;       raster synchronization (having the screen turned off) #2

lce25   bit     $d011
        bmi     lce25

        lda     $dd00
lce2d   ora     #$dc
        pha

        ;       VDOS <- sending drive code to the drive

        ldy     #$9b
        ;ldx     #$00

        jsr     vload+$c600
        ldx     #$02

        ;       VDOS <- loading loop

lcvlp   inx
        pla
        inx
        pha
        stx     $aa

        and     #$17
        sty     $a2
        tax

        ;       (timeout: > 1 sec)

lctmp   cli
        bit     $a2
        sei
        sec
        bvs     lcesc

        bit     $dd00
        bmi     lctmp

        stx     $dd00

        and     #$07
        sta     $a7

lcqlp   beq     lcvlp

;        ;       Raster synchronization (badline check):
;
;lbadl   lda     $d012
;        eor     #$02
;        and     #$06
;
;        ;       (*+1 = $cexx) <- $f7: beq lbadl (badline check: yes)
;        ;                        $00: beq lnbdl (badline check: no)
;
;        ;       (see below at the end of the code for self-modification!)
;
;;lbdc1   beq     lbadl
;lbdc1   beq     lnbdl

lnbdl   ;lda     $a7
        sta     $dd00
        sei
        pla
        pha
        pla
        pha
        lda     $dd00
        sei
        lsr
        lsr
        ora     $dd00
        and     #$f0
        sta     laddc+$c601
        lda     $dd00
        sei
        lsr
        lsr
        ora     $dd00
        stx     $dd00
        lsr
        lsr
        lsr
        lsr

laddc   ora     #$00
        sta     $cd00,y
        lda     $a7
        iny

        ;       (*+1 = $cexx) <- $c7: bne lbadl (badline check: yes)
        ;                        $d0: bne lnbdl (badline check: no)

        ;       (see below at the end of the code for self-modification!)

;lbdc2   bne     lbadl
lbdc2   bne     lnbdl

        ;lda     $a7
        sta     $dd00

        ;       turn on fast mode + control data pointer (set forth)

        jsr     $cc10
        jsr     $ced4

        ;       (*+1 = $ce91) <- write into RAM everywhere ($35: + I/O area on)

        lda     #$34
        ldx     $aa
        sta     $01

        lda     $cd00
        sec
        bne     lcemc

        dex

lclp0   lda     $cd01,x
        inx
        jsr     $cece
        cpx     $cd01
        bcc     lclp0

        clc
lcesc   ldx     #$80

lcemc   bmi     lcepc

lclp1   lda     $cd00,x
        jsr     $cece
        inx
        bne     lclp1

lcepc   php

        ;       control data pointer (set back) + turn off fast mode

        jsr     $cef8
        jsr     $cf0b

        plp
        beq     lcqlp

        ;       (* = $cec1)

        ;       VDOS/SJLOAD (common) <- end of loading (restore $dd00 + close)

lcec1   pla
        sta     $dd00
        bne     lcf2c

        ;       (* = $cec7)

lcec7   sei

        ;       (*+1 = $cec9) <- write into RAM everywhere ($35: + I/O area on)

lcec8   ldx     #$34
        stx     $01

lcecc   ldx     #$36

        ;       (* = $cece) <- increment data pointer

lcece   sta     ($ae),y

lced0   inc     $ae
        beq     lcedb

        ;       control data pointer ($ca00-$cfca and $fa00-$ffca swapped!)
        ;       (or $cc00/$fc00 if no swapping)

lced4   lda     $af

        ;       (*+1 = $ced7)

        cmp     #$ca
        bcs     lcedf

        rts

lcedb   inc     $af
        bne     lced4

lcedf   cmp     #$ff
        lda     $ae
        bcc     lcee9

        cmp     #$cb
        bcs     lcef1

lcee9   cmp     #$cb
        lda     $af
lceed   sbc     #$cf
        bcs     lcef7

lcef1   lda     $af

lcef3   eor     #$30
        sta     $af

lcef7   rts

        ;       control data pointer ($ca00-$cfca and $fa00-$ffca swapped!)
        ;       (or $cc00/$fc00 if no swapping)

lcef8   lda     $af

        ;       (*+1 = $cefb)

        cmp     #$fa
        bcs     lcef3

        rts

        ;       (* = $ceff) <- turn off fast mode + save registers

lceff   lda     $d020
        sta     $cf9e

        lda     $d011
        sta     $cfa8

        ;       (* = $cf0b) <- turn off fast mode

lcf0b   lda     #$37
        sta     $01

        ;       (*+1 = $cf10) <- $ce24 (RTS) <-> $ce09 (on C65/MEGA65 or DTV)

lcf0f   jsr     $ce24
        ;jsr     $ce21
        ;sty     $d02f

        ;       (* = $cf12)

lcf12   lda     $d030
        sty     $d030
        sty     $d07a

lcf1b   rts

        ;       (* = $cf1c) <- (listen to command channel)

lcf1c   lda     $ba
        jsr     $ffb1
        lda     #$6f
        jmp     $ff93

        ;       (* = $cf26) <- (untalk + close)

lcf26   jsr     $ffab
        jmp     $f642

        ;       close on success (otherwise retry)

lcf2c   bcs     lcf1b

        pla
        pla

        ;       (* = $cf30) <- (return from load routine with close)

lcf30   jsr     $cf26

        ;       (* = $cf33) <- send "i" to device after loading (if no error)
        ;       (needed by VDOS to get rid of the loader code in drive)

lcf33   jsr     $cf1c
        lda     #$49
        jsr     $ffa8
        jsr     $ffae

        ;       (* = $cf3e)

        sei
        clc

        ;       (* = $cf40)

        ;       return from load routine and/or
        ;       restore screen + fast mode (before Kernal call) etc.

lcf40   pha

        ;       EOI cleared on status (checking for an error)

        lda     #$be
        ;lda     #$bf
        ;lda     #$ff
        and     $90
        sta     $90
        beq     lcf4a

        sec

        ;       if succeeded, stores the last used device number ($a8 -> $a9),
        ;       but if failed, then reverts to the former ($a8 <- $a9)

lcf4a   lda     $a8
        bcc     lcf52

        lda     $a9

lcf50   sta     $a8
lcf52   sta     $a9

        php
        jsr     $f32f
        ;jsr     $ffe7
        plp

        ;       release IEC bus (on C64)
        ;       (POKE 56576,199)

        lda     $dd00
        and     #$03
        ora     #$c4
        sta     $dd00

        jsr     $cfb1

        ;       (*+1 = $cf67) <- CRC of $02b0-$02ff + $ca6f-$cabe + swap r.

        eor     #$00
        beq     swend

        pla
        tay

        ;       (* = $cf6c)

        pla
        pla

        tya
        pha

        txa
        ldy     #>$cc29
        ldx     #<$cc29

        ;       (*+1 = $cf76) <- CRC of swap routine ($ffcb-$fff5)

        eor     #$00
        stx     $cf6c
        bne     swenc

swexx   lda     $cf87,x
        dex
        sta     $0100,x
        bne     swexx

        jmp     $0100

        ;       (* = $cf88)

swaff   jsr     $ffcb

        ;       (*+1 = $cf8c) <-> (*+3 = $cf8e) <- Kernal LOAD vector (orig)

lcf8b   ldx     #<$f4a5
lcf8d   ldy     #>$f4a5

        ;       (* = $cf8f)

swenc   stx     $0330
        sty     $0331

        ;       (* = $cf95) <- return from load routine

swend   lda     #$37
        ldx     $ae
        ldy     $af
        sta     $01

        ;       restore screen + fast mode (before Kernal call)

        ;       (*+1 = $cf9e)

        lda     #$00
        sta     $d020

        ;       (*+1 = $cfa3)

lcfa2   lda     #$00
        sta     $d030

        ;       (*+1 = $cfa8)

        lda     #$1b
        and     #$7b
        sta     $d011

        pla
        cli

        ;       (* = $cfb0)

        rts

        ;       (* = $cfb1) <- make CRC

lcfb1   sei

        lda     #$34
        ldx     #$2a
        ldy     #$4f
        sta     $01

swex    eor     $cfcb,x
        dex
        bpl     swex

        tax

swfx    eor     $02b0,y
        eor     l600+$c600,y
        dey
        bpl     swfx

        ;       (* = $ffca) <- (* = $cfca)

        rts

        ;       (* = $09cb) <- (* = $cfcb)

        ;       ----------------------------------------
        ;       SJLOAD/VDOS helper ($ffcb-$fff5)
        ;       ----------------------------------------

        ;       (* = $ffcb) swapping $ca00-$cfca and $fa00-$ffca
        ;       (also for swapping $c000-$cfff and $f000-$ffff)

swp     ldy     #$00

swc     lda     #$c0
        sty     $0c
        sta     $0d

swd     lda     #$f0
        sty     $0e
        sta     $0f

swy     lda     ($0c),y
        pha
        lda     ($0e),y
        sta     ($0c),y
        pla
        sta     ($0e),y

        tya
        iny
        bne     swync

        inc     $0d
        inc     $0f

swync   eor     #$ff
        bne     swy

swynd   lda     #$d0

        eor     $0d
        bne     swy

        rts

        ;       (* = $fff6) <- (* = $cff6)

        plp
        sei
        php
        rti

        ;       (* = $fffa) <- (* = $cffa)

        word    $fff6
        word    $fff6
        word    $fff6

        ;       (* = $0a00) <- (* = $d000)

*=$0a00

        ;       (* = $0a00) <- (* = $0edb)

        ;       compiled with CBM prg Studio v3.13 (by Arthur Jordison)
        ;       http://ajordison.co.uk/

        ;       Rosetta Interactive Fiction project homepage:

        ;       http://istennyila.hu/rosetta

        ;       v0.13 (c) 2012-2023 by Robert Olessak

        ;       SDOS project page:

        ;       http://istennyila.hu/sdos
